// packages/core/src/index.ts
import { useCallback as useCallback2, useEffect, useRef, useState } from "react";

// packages/core/src/handlers.ts
import { useCallback } from "react";
function usePush(router) {
  return useCallback(
    (href) => {
      router.push(href);
    },
    [router]
  );
}
function useHandlers(push, stateRef) {
  const reset = useCallback(() => {
    const href = `?${new URLSearchParams({}).toString()}`;
    push(href);
  }, [push]);
  const replace = useCallback(
    (data) => {
      const href = `?${new URLSearchParams(data).toString()}`;
      push(href);
    },
    [push]
  );
  const setValue = useCallback(
    (key, value) => {
      const href = `?${new URLSearchParams({
        ...stateRef.current.data,
        [key]: value
      }).toString()}`;
      push(href);
    },
    [push, stateRef]
  );
  return { reset, replace, setValue };
}

// packages/core/src/router.ts
var UrlStateRouter = class {
  push(href) {
  }
  onSearchParamsChange() {
  }
  destroy() {
  }
};
var genericRouterCurrentStateString = "";
var GenericRouter = class extends UrlStateRouter {
  constructor(callback) {
    super();
    this.callback = callback;
    this.interval = 0;
    if (typeof window !== "undefined") {
      this.interval = setInterval(() => {
        this.onSearchParamsChange();
      }, 100);
    }
  }
  push(href) {
    window.history.pushState({}, "", href);
    this.onSearchParamsChange();
  }
  onSearchParamsChange() {
    if (window.location.search !== genericRouterCurrentStateString) {
      genericRouterCurrentStateString = window.location.search;
      this.callback(window.location.search);
    }
  }
  destroy() {
    if (this.interval) {
      clearInterval(this.interval);
    }
  }
};

// packages/core/src/utils.ts
function urlParamsToObject(params) {
  const object = {};
  for (const [key, value] of params.entries()) {
    if (!object[key]) {
      object[key] = value;
      continue;
    }
    const currentValue = object[key];
    if (typeof currentValue === "string") {
      object[key] = [currentValue, value];
    } else {
      currentValue.push(value);
    }
  }
  return object;
}

// packages/core/src/index.ts
var subscribers = /* @__PURE__ */ new Map();
var currentStateString = "";
function update(newSearchParams) {
  console.log("Update");
  if (newSearchParams !== currentStateString) {
    currentStateString = newSearchParams;
    subscribers.forEach((subscriber) => subscriber(currentStateString));
  }
}
function subscribeToHistory(fn) {
  subscribers.set(fn, fn);
}
function unsubscribeFromHistory(fn) {
  subscribers.delete(fn);
}
function useUrlState(schema, options) {
  const [router] = useState(new GenericRouter(update));
  useEffect(() => {
    return () => {
      router.destroy();
    };
  }, [router]);
  return useUrlStateWithRouter(schema, {
    ...options,
    preserveUnknown: false,
    router
  });
}
function useUrlStateWithRouter(schema, options) {
  const schemaRef = useRef(schema);
  const [state, setState] = useState({
    data: null,
    isError: false,
    error: null,
    isReady: false
  });
  const stateRef = useRef(state);
  stateRef.current = state;
  const recalculateState = useCallback2(
    (searchString) => {
      const params = new URLSearchParams(searchString);
      const object = urlParamsToObject(params);
      const validationResult = schemaRef.current.safeParse(object);
      const result = validationResult.success ? { success: true, data: validationResult.data, error: null } : { success: false, data: null, error: validationResult.error };
      if (options?.preserveUnknown) {
        result.data = { ...object, ...result.data };
      }
      setState({
        data: result.data,
        isError: !result.success,
        error: result.error,
        isReady: true
      });
    },
    [options?.preserveUnknown]
  );
  useEffect(() => {
    subscribeToHistory(recalculateState);
    return () => {
      unsubscribeFromHistory(recalculateState);
    };
  }, [recalculateState]);
  useEffect(() => {
    update(window.location.search);
  }, []);
  const push = usePush(options.router);
  const handlers = useHandlers(push, stateRef);
  return { ...state, ...handlers };
}
export {
  useUrlState,
  useUrlStateWithRouter
};
